import { Workflow } from "../workflow-types";
import { TEMPLATE_OWNER } from "./template-owner";
import { QUICKSTART_WELCOME_WORKFLOW } from "./quickstart-welcome";
import { SIMPLE_PYTHON_WORKFLOW } from "./python-task";
import { FEEDLY_DIGEST_WORKFLOW } from "./feedly-digest";
import { SLACK_BROADCAST_WORKFLOW } from "./slack-broadcast";
import { RSS_MONITOR_WORKFLOW } from "./rss-monitor";
import { MONGODB_SESSION_WORKFLOW } from "./mongodb-session";
import { TELEGRAM_BROADCAST_WORKFLOW } from "./telegram-broadcast";

export const SAMPLE_WORKFLOWS: Workflow[] = [
  QUICKSTART_WELCOME_WORKFLOW,
  SIMPLE_PYTHON_WORKFLOW,
  FEEDLY_DIGEST_WORKFLOW,
  SLACK_BROADCAST_WORKFLOW,
  RSS_MONITOR_WORKFLOW,
  MONGODB_SESSION_WORKFLOW,
  TELEGRAM_BROADCAST_WORKFLOW,
];

export {
  TEMPLATE_OWNER,
  QUICKSTART_WELCOME_WORKFLOW,
  SIMPLE_PYTHON_WORKFLOW,
  FEEDLY_DIGEST_WORKFLOW,
  SLACK_BROADCAST_WORKFLOW,
  RSS_MONITOR_WORKFLOW,
  MONGODB_SESSION_WORKFLOW,
  TELEGRAM_BROADCAST_WORKFLOW,
};
